%  This program computes a single realization of the stochastic version of 
%  Jones and Manuelli's model through a log-linear approximation, as 
%  described in the corresponding section of chapter 5. 
%  The deterministic version of the model can be solved by choosing sigmae = 0.0. 

%  The AK model can be obtained as a special case by choosing B = .0, as
%  explained in the last section of chapter 5.

%  @Copyright J. Ruiz (2021).

clear
nobs = 300; % number of observations

%  structural parameters

A=1.0;              %    technology parameter
B=1.0;              %    technology parameter
beta= 0.95;      	%    discount factor
sigma=3.0;          %    inverse of the intertemporal elasticity of substitution 
%                        (relative risk aversion)
n=0.01;             %    population growth rate
delta=0.1;          %    depreciation rate
alpha=0.35;         %    capital share
tauc=0.12;          %    consumption tax rate
tauy=0.2;           %    income tax rate
rho=0.95;           %    autoregressive parameter
sigmae=0.01;        %    standard deviation of technology innovation

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% steady state

gss = ((beta/(1+n))*((1-tauy)*A+1-delta))^(1/sigma);
ckss = (1/(1+tauc))*((1-tauy)*A+1-delta-(1+n)*gss);
xss = ckss;
ykss = A;  
zss = ykss;
thetass = 1;
kss = 1;

phi1 = beta*(gss^(1-sigma));
phi2 = (sigma*(gss^(sigma-1))-beta*alpha*(gss^(alpha-1)))*...
       (1-tauy)*A*beta*(gss^(1-2*sigma))/((1+n)*sigma);
phi3 = -(beta^2)*((1-tauy)*A/(sigma*(1+n)))*(gss^(1-2*sigma))*...
       (rho-(gss^(alpha-1)));

psi2 = (sigma*(gss^(sigma-1))-beta*rho)*...
       (1-tauy)*A*beta*(gss^(1-2*sigma))/((1+n)*sigma);
   
% innovation

aleatn = normrnd(0,1,nobs+200,1);
eps = sigmae*aleatn(201:length(aleatn))';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

k0 = kss; 
theta0 = thetass; ltheta0=log(theta0);
z0 = theta0*(A+B*k0^(alpha-1));
zc0=log(z0/zss);
if B>0
    xc0=(phi2/(1-phi1*(gss^(alpha-1))))*zc0+ltheta0*...
        (((phi2+phi3)/(1-phi1*rho))-(phi2/(1-phi1*(gss^(alpha-1)))));
end
if B==0
    xc0=psi2*ltheta0/(1-rho*phi1);
end

z=[]; x=[]; lk=[]; lc=[]; ly=[]; li=[]; ktl=[]; ctl=[]; ytl=[]; itl=[];
ltheta=[];

ktl=[ktl;k0]; lk=[lk;log(k0)];
ltheta=[ltheta;ltheta0];
z=[z;z0];

for t=1:nobs
    
    lthetat=rho*ltheta0+eps(t);
    ltheta0=lthetat;
    ltheta=[ltheta;lthetat];
    
    if B>0
        zct=(gss^(alpha-1))*zc0+lthetat-(gss^(alpha-1))*ltheta0;
        zc0=zct;
        ltheta0=lthetat;
        zt=zss*exp(zct);
        z=[z;zt];
    
        xct=(phi2/(1-phi1*(gss^(alpha-1))))*zct+lthetat*...
            (((phi2+phi3)/(1-phi1*rho))-(phi2/(1-phi1*(gss^(alpha-1)))));
        xt=xss*exp(xct);
        x=[x;xt];
    end
    if B==0;
        zct=lthetat;
        zt=zss*exp(zct);
        z=[z;zt];
        
        xct=psi2*lthetat/(1-rho*phi1);
        xt=xss*exp(xct);
        x=[x;xt];
    end
        
    ktlt=(((1-tauy)*zt+1-delta-xt*(1+tauc))/(1+n))*k0;
    k0=ktlt;
    ktl=[ktl;ktlt];
    
    lkt=-t*log(gss)+log(ktlt);
    lk=[lk;lkt];
    
    ctlt=ktlt*xt;
    ctl=[ctl;ctlt];
    
    lct=lkt+log(xt);
    lc=[lc;lct];
        
    ytlt=exp(lthetat)*(A*ktlt+B*(ktlt^alpha));
    ytl=[ytl;ytlt];
    
    lyt=log(A*exp(lkt)+B*((gss^(alpha-1))^t)*exp(alpha*lkt))+lthetat;
    ly=[ly;lyt];
end

% investment
for t=1:nobs
    itlt=(1+n)*ktl(t+1)-(1-delta)*ktl(t);
    itl=[itl;itlt];
    
    lit=-t*log(gss)+log(itlt);
    li=[li;lit];
end



revy=tauc.*(x./(z(1:length(x)).*exp(ltheta(1:length(x)))))+tauy;

gc=ctl(2:length(ctl))./ctl(1:length(ctl)-1);
gk=ktl(2:length(ktl))./ktl(1:length(ktl)-1);
gy=ytl(2:length(ytl))./ytl(1:length(ytl)-1);
gi=itl(2:length(itl))./itl(1:length(itl)-1);

V=[lc(1:length(gi)),lk(1:length(gi)),ly(1:length(gi)),li(1:length(gi)),...
    revy(1:length(gi)),gc(1:length(gi)),gk(1:length(gi)),gy(1:length(gi)),...
    gi(1:length(gi))];

figure;
hold on

subplot(4,3,1)
plot(lc(1:length(x)));
title('consumption (cycle)');
xlabel('time');

subplot(4,3,2)
plot(ly(1:length(x)));
title('output (cycle)');
xlabel('time');

subplot(4,3,3)
plot(lk(1:length(x)));
title('capital stock (cycle)');
xlabel('time');

subplot(4,3,4)
plot(li(1:length(x)));
title('investment (cycle)');
xlabel('time');

subplot(4,3,5)
plot(revy(1:length(x)));
title('public revenues to output ratio');
xlabel('time');

subplot(4,3,6)
plot(ltheta(1:length(x)));
title('productivity shock');
xlabel('time');

subplot(4,3,7)
plot(exp(lc(1:length(x))-ly(1:length(x))));
title('consumption to output ratio');
xlabel('time');

subplot(4,3,8)
plot(exp(li(1:length(x))-ly(1:length(x))));
title('investment to output ratio');
xlabel('time');

subplot(4,3,9)
plot(gc(1:length(gi)));
title('consumption (growth rate)');
xlabel('time');

subplot(4,3,10)
plot(gk(1:length(gi)));
title('stock of capital (growth rate)');
xlabel('time');

subplot(4,3,11)
plot(gy(1:length(gi)));
title('output (growth rate)');
xlabel('time');

subplot(4,3,12)
plot(gi(1:length(gi)));
title('investment (growth rate)');
xlabel('time');

hold off

        