% This program computes multiple realizations of the endogenous growth model
% with human capital accumulation, which may include an externality in the 
% production of the final good through the average stock of human capital,
% so long as the psi-parameter is positive.
% If the psi-parameter is equal to zero, we are back at the model without
% externalities.
%
% The program uses the full Blanchard-Kahn approach by representing the model in
% state-space form.
%
% Two benchmark parameterizations are given, one for the case of a determinate 
% equilibrium, with another parameterization for the case of indeterminacy 
% of equilibria:
%
% Benchmark parameters under determinacy:
% A=1; B=0.0237; beta=0.99; sigma=1.5; n=0.0035; deltak=0.025;
% deltah=0.008;alpha=0.36;psi=0;
%
% Benchmark parameters under indeterminacy:
% A=1; B=0.08; beta=0.92; sigma=0.23; n=0.0035; deltak=0.025;
% deltah=0.008;alpha=0.36;psi=0.5;

%  @Copyright J. Ruiz 2007 

clear

%  structural parameters
A=1.0;              %    technology level, final good sector
B=0.0237;           %    technology level, educational sector
beta=0.99;          %    discount factor
sigma=1.5;          %    inverse of the intertemporal elasticity of substitution (relative risk aversion)
n=0.0035;          	%    population growth rate
deltak=0.025;      	%    depreciation rate of physical capital
deltah=0.008;      	%    depreciation rate of human capital
alpha=0.36;         %    physical capital share (final good sector)
phi1=0.95;          %    persistence parameter of stochastic process for shock in sector of final good
phi2=0.95;          %    persistence parameter of stochastic process for shock in educational sector

psi=0;              %    average human capital share (final good sector)

nobs=150;           %    number of observations

sigmae1=0.001;       %    standard deviation innovation 1
sigmae2=0.001;        %    standard deviation innovation 2
corr12 = .0;        %    linear correlation between the two productivity innovations 

%parameters to draw a forecast error under equilibria indeterminacy
a=0.0;
b=0.0;
sigmae=0.01;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %  steady state
    gamma=(beta*(B+1-deltah))^((1-alpha+psi)/(sigma*(1-alpha+psi)-psi))-1;
    gammah=(1+gamma)^((1-alpha)/(1-alpha+psi))-1; sigmat=sigma*(1-alpha+psi)/(1-alpha);
    u=1-((gammah+deltah)/B); 
            u0=0.42; % initial condition for a control variable 
    z=(A*alpha*(u^(1-alpha))/((((1+gamma)^sigma)*(1+n)/beta)-(1-deltak)))^(1/(1-alpha));
    x=A*z^(alpha-1)*u^(1-alpha)+(1-deltak)-(1+n)*(1+gamma);
              param=[A,B,beta,sigma,n,deltak,deltah,alpha,phi1,phi2,psi];
              luss0=log(u);lxss0=log(x);lzss0=log(z);lthetss0=0;letass0=0;
              X0=[lzss0 lzss0 lxss0 lxss0 luss0 luss0 lthetss0 lthetss0 letass0 letass0]';
              X1=fsolve('ssendog',X0,optimset,param);
              lzss=X1(1);zss=exp(lzss);
              lxss=X1(3);xss=exp(lxss);
              luss=X1(5);uss=exp(luss);
    
              J=jacobian('ssendog',X1,param);
                GAMMA0 = -[J(1,1) J(1,3) J(1,5);
                          J(2,1) J(2,3) J(2,5);
                          J(3,1) J(3,3) J(3,5)];
                GAMMA1 = [J(1,2) J(1,4) J(1,6);
                         J(2,2) J(2,4) J(2,6);
                         J(3,2) J(3,4) J(3,6)];
                GAMMA2 = [J(1,8)+phi1*J(1,7) J(1,10)+phi2*J(1,9);
                         J(2,8)+phi1*J(2,7) J(2,10)+phi2*J(2,9);
                         J(3,8)+phi1*J(3,7) J(3,10)+phi2*J(3,9)];
                     
                GAMMA3=inv(GAMMA0)*GAMMA1;
                GAMMA4=inv(GAMMA0)*GAMMA2;
             
  [evec,eval]=eig(GAMMA3);
  
  eval2=diag(eval);

  [eval3,ord]=sort(abs(eval2));
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  paramet=[A,B,beta,sigma,n,deltak,deltah,alpha,phi1,phi2,psi,nobs,...
      sigmae1,sigmae2,a,b,sigmae,u0];
  
    nreal=10;       % simulations
    Mmean=[];       % matrix of means
    Mstd=[];        % matrix of standard desviations 
    Mcv=[];         % matrix of coefficients of variation 
    Mcorr=[];       % matrix of correlations with output 
    Mcorr1=[];      % matrix of correlations with output(+1) 
    Mcorr2=[];      % matrix of correlations with output(+2) 
    Mcorrm1=[];     % matrix of correlations with output(-1) 
    Mcorrm2=[];     % matrix of correlations with output(-2)
    Mcorrpu=[];     % correlation between labor productivity and worked hours
  
  
  for j=1:nreal
      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      % innovations
      aleatn=normrnd(0,1,nobs+200,3);

      eps1=sigmae1*aleatn(201:length(aleatn),1);
      eps2=sigmae2*aleatn(201:length(aleatn),2);

      % If the economy displays indeterminacy then we must draw a sample 
      % realization for a forecast error (for example: error=a*eps1+b*eps2+eps;
      % eps iid N(0,sigmae); corr(error,eps1)=a*sigmae1/sigmae2;
      % corr(error,eps2)=b*sigmae2/sigmae1);
      % Parameters a,b,sigmae1 and sigmae2 determine correlations between
      % forecast error and each innovation. The program will then call 
      % function simul2.m, while function simul1.m is called in the case
      % of a determined equilibrium
      sigmaerr=sqrt((a^2)*sigmae1^2+(b^2)*sigmae2^2+sigmae^2);
      error=a*eps1+b*eps2+sigmae*aleatn(201:length(aleatn),3);
    
        if eval3(2)>1
            V=simul1(paramet,eps1,eps2);
        end
        
        if eval3(2)<1
            V=simul2(paramet,eps1,eps2,error);
        end
        
            lc=log(V(:,1));
            lk=log(V(:,2));
            lh=log(V(:,3));
            u=V(:,4);
            ly=log(V(:,5));
            lq=log(V(:,6));
            prod=V(:,7);
            gammah=V(:,8);
            gammak=V(:,9);
            gammac=V(:,10);
            gammay=V(:,11);
            gammaq=V(:,12);
            
            % Hodrick&Prescott Filter
    
            clc=lc-hp(length(lc),1600)*lc;
            clk=lk-hp(length(lc),1600)*lk;
            clh=lh-hp(length(lc),1600)*lh;
            cly=ly-hp(length(lc),1600)*ly;
            clq=lq-hp(length(lc),1600)*lq;
            
            V(:,1)=exp(clc+mean(lc)+.5*var(clc));
            V(:,2)=exp(clk+mean(lk)+.5*var(clk));
            V(:,3)=exp(clh+mean(lh)+.5*var(clh));
            V(:,5)=exp(cly+mean(ly)+.5*var(cly));
            V(:,6)=exp(clq+mean(lq)+.5*var(clq));
            c=V(:,1);
            k=V(:,2);
            h=V(:,3);
            y=V(:,5);
            q=V(:,6);
%             lc=clc;
%             lk=clk;
%             ly=cly;
%             lq=clq;
%             lh=clh;
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
            T=length(V(:,1));
    
            Mmean=[Mmean; mean(V)];
            Mstd=[Mstd; std(V)];
            Mcv=[Mcv; 100.*std(V)./mean(V)];          
                RHOCY=corrcoef(clc,cly);
                RHOHY=corrcoef(clh,cly);
                RHOKY=corrcoef(clk,cly)
                RHOQY=corrcoef(clq,cly);
                RHOCQ=corrcoef(clc,clq);
                RHOGKY=corrcoef(gammak,gammay);
                RHOGHY=corrcoef(gammah,gammay);
                RHOGCY=corrcoef(gammac,gammay);
                RHOGQY=corrcoef(gammaq,gammay);
                RHOGCQ=corrcoef(gammac,gammaq);
                RHOPU=corrcoef(prod,u);
            Mcorr=[Mcorr; RHOCY(1,2) RHOHY(1,2) RHOKY(1,2) RHOQY(1,2)...
                RHOCQ(1,2) RHOGCY(1,2) RHOGHY(1,2) RHOGKY(1,2) RHOGQY(1,2)...
                RHOGCQ(1,2) RHOPU(1,2)];   
    
                
  end
       

    AVGm=mean(Mmean);
    AVGs=mean(Mstd);
    AVGcv=mean(Mcv);
    AVGcorr=mean(Mcorr);
    
    STDm=std(Mmean)./(nreal^0.5);
    STDs=std(Mstd)./(nreal^0.5);
    STDcv=std(Mcv)./(nreal^0.5);
    STDcorr=std(Mcorr)./(nreal^0.5);
    
    
display '_______________________________________________________________________________________________________________ '
display 'Average and standard deviation of time series mean across simulations '
[AVGm;STDm]    
display '      c       k         h        u        y       q       prod       gh        gk       gc       gy      gq'

display '_______________________________________________________________________________________________________________ '
display 'Time series standard deviations (average and standard deviation across simulations)'
[AVGs;STDs]    
display '      c       k         h        u        y       q       prod       gh        gk       gc       gy      gq'

display '_______________________________________________________________________________________________________________ '
display 'Coefficients of variation (average and standard deviation of the coefficient of variation across simulations)'
[AVGcv;STDcv]    
display '      c       k         h        u        y       q       prod       gh        gk       gc       gy      gq'

display '_______________________________________________________________________________________________________________ '
display 'Linear correlation coefficients (average and standard deviation across simulations)'
[AVGcorr;STDcorr]    
display '    lc,ly     lh,ly      lk,ly     lq,ly    lc,lq     gc,gy    gh,gy     gk,gy     gq,gy    gc,gq    prod,u'

display '_______________________________________________________________________________________________________________ '

save means.prn AVGm STDm -ascii   
save stds.prn AVGs STDs -ascii
save cvs.prn AVGcv STDcv -ascii
save cys.prn AVGcorr STDcorr -ascii

    figure;
    hold on
    
    subplot(3,2,1)
    plot(c);
    title('Consumption');
    xlabel('time');

    subplot(3,2,2)
    plot(y);
    title('Output');
    xlabel('time');

    subplot(3,2,3)
%    plot([lk,ones(length(lk),1)*kss]);
    plot(k);
    title('Stock of physical capital');
    xlabel('time');
    
    subplot(3,2,4)
    plot(h);
    title('Stock of human capital');
    xlabel('time');

    subplot(3,2,5)
    plot(prod);
    title('Productivity');
    xlabel('time');

    subplot(3,2,6)
    plot(u);
    title('Hours worked');
    xlabel('time');
    
    hold off

