% This program computes a single realization of the endogenous growth model
% with human capital accumulation, which may include an externality in the 
% production of the final good through the average stock of human capital,
% so long as the psi-parameter is positive.
% If the psi-parameter is equal to zero, we are back at the model without
% externalities.

% The program uses the full Blanchard-Kahn approach by representing the model in
% state-space form.

% Two benchmark parameterizations are given, one for the case of a determinate 
% equilibrium, with another parameterization for the case of indeterminacy 
% of equilibria:
% Benchmark parameters under determinacy:
% A=1; B=0.0237; beta=0.99; sigma=1.5; n=0.0035; deltak=0.025;
% deltah=0.008;alpha=0.36;psi=0;
% Benchmark parameters under indeterminacy:
% A=1; B=0.08; beta=0.92; sigma=0.23; n=0.0035; deltak=0.025;
% deltah=0.008;alpha=0.36;psi=0.5;

%  @Copyright J. Ruiz 2007 

clear

%  Structural parameters

A=1.0;              %    technology level, final good sector
B=0.0237;           %    technology level, educational sector
beta=0.99;	        %    discount factor
sigma=1.5; 	        %    inverse of the intertemporal elasticity of substitution (relative risk aversion)
n=0.0035;           %    population growth rate
deltak=0.025;       %    depreciation rate of physical capital
deltah=0.008;       %    depreciation rate of human capital
alpha=0.36;         %    physical capital share (final good sector)
phi1=0.95;          %    persistence parameter of stochastic process for shock in sector of final good
phi2=0.95;          %    persistence parameter of stochastic process for shock in educational sector

psi=0.2;              %    elasticity of average human capital (final good sector)

%  Structural parameters

% A=1.0;              %    technology level, final good sector
% B=0.08;             %    technology level, educational sector
% beta=0.92;	      %    discount factor
% sigma=.23; 	      %    inverse of the intertemporal elasticity of substitution (relative risk aversion)
% n=0.0035;           %    population growth rate
% deltak=0.025;       %    depreciation rate of physical capital
% deltah=0.008;       %    depreciation rate of human capital
% alpha=0.36;         %    physical capital share (final good sector)
% phi1=0.95;          %    persistence parameter of stochastic process for shock in sector of final good
% phi2=0.95;          %    persistence parameter of stochastic process for shock in educational sector
% 
% psi=0.5;            %    elasticity of average human capital (final good sector)

nobs=150;           %    number of observations

sigmae1=0.001;      %    standard deviation innovation 1
sigmae2=0.001;      %    standard deviation innovation 2

%  Activate the next two lines when computing impulse responses to a shock in either sector 
%  sigmae1=0.00;      %    standard deviation innovation 1
%  sigmae2=0.00;      %    standard deviation innovation 2

% Innovations
   aleatn=normrnd(0,1,nobs+200,3);

   eps1=sigmae1*aleatn(201:length(aleatn),1);
   eps2=sigmae2*aleatn(201:length(aleatn),2);

%   eps1(10) = .01;  %  Activate when computing responses to an impulse in the productivity of the final good sector
%   eps2(10) = .01;  %  Activate when computing responses to an impulse in the productivity of the educational sector


% If the economy displays indeterminacy then we must draw realizations for 
% a forecast error (for instance: error=a*eps1+b*eps2+eps;
% eps iid N(0,sigmae); corr(error,eps1)=a*sigmae1/sigmae2;
% corr(error,eps2)=b*sigmae2/sigmae1);
% The choice of values for a,b,sigmae1 and sigmae2 determines the correlations 
% between the forecast error and each innovation
    a=0.0;b=0.0;sigmae=0.001;
    sigmaerr=sqrt((a^2)*sigmae1^2+(b^2)*sigmae2^2+sigmae^2);
    
    error=a*eps1+b*eps2+sigmae*aleatn(201:length(aleatn),3);
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%  Steady state

    gamma=(beta*(B+1-deltah))^((1-alpha+psi)/(sigma*(1-alpha+psi)-psi))-1;
    gammah=(1+gamma)^((1-alpha)/(1-alpha+psi))-1; sigmat=sigma*(1-alpha+psi)/(1-alpha);
    u=1-((gammah+deltah)/B);
        u0=u; % Choice of initial condition for a control variable to determine a solution under indeterminacy of equilibria 
    z=(A*alpha*(u^(1-alpha))/((((1+gamma)^sigma)*(1+n)/beta)-(1-deltak)))^(1/(1-alpha));
    x=A*z^(alpha-1)*u^(1-alpha)+(1-deltak)-(1+n)*(1+gamma);

         
    GAMMA0=[(1-alpha+sigma)*((1+n)/beta)*(1+gamma)^sigma-(1-alpha)*(1-deltak)     sigma*((1+n)/beta)*(1+gamma)^sigma                          -(1-alpha)*((((1+n)/beta)*(1+gamma)^sigma)-(1-deltak));
            -(alpha-sigma)*((1+gamma)^sigma)/(beta*(1+gammah)^(psi/(1-alpha)))   sigma*((1+gamma)^sigma)/(beta*(1+gammah)^(psi/(1-alpha)))   alpha*((1+gamma)^sigma)/(beta*(1+gammah)^(psi/(1-alpha)));
            (1+n)*(1+gamma)*z                                                    0                                                           0];
        
    GAMMA1=[sigma*((1+n)/beta)*(1+gamma)^sigma                                   sigma*((1+n)/beta)*(1+gamma)^sigma                          sigmat*(1+n)*B*u*((1+gamma)^sigma)/(beta*(1+gammah));
            -(alpha-sigma)*((1+gamma)^sigma)/(beta*(1+gammah)^(psi/(1-alpha)))   sigma*((1+gamma)^sigma)/(beta*(1+gammah)^(psi/(1-alpha)))   alpha*((1+gamma)^sigma)/(beta*(1+gammah)^(psi/(1-alpha)))+(sigmat-(psi/(1-alpha)))*((1+gamma)^(sigma-1))*B*u/beta;
            ((((1+n)/beta)*(1+gamma)^sigma)-x)*z                                 -z*x                                                        (1+n)*z*((1+gamma)/(1+gammah))*((1-alpha+psi)/(1-alpha))*B*u+(1-alpha)*A*(z^alpha)*(u^(1-alpha))];
        
    GAMMA2=[((((1+n)/beta)*(1+gamma)^sigma)-(1-deltak))*phi1                     -sigmat*(1+n)*B*(1-u)*((1+gamma)^sigma)/(beta*(1+gammah));
            -((1+gamma)^sigma)/(beta*(1+gammah)^(psi/(1-alpha)))*(1-phi1)        ((1+gamma)^sigma)/(beta*(1+gammah)^(psi/(1-alpha)))-(sigmat-(psi/(1-alpha)))*((1+gamma)^(sigma-1))*B*(1-u)/beta-(1-deltah)*phi2;
            A*(z^alpha)*(u^(1-alpha))                                            -(1+n)*z*B*(1-u)*((1+gamma)/(1+gammah))*((1-alpha+psi)/(1-alpha))];

    GAMMA3=inv(GAMMA0)*GAMMA1;
    
    GAMMA4=inv(GAMMA0)*GAMMA2;    

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%%%%%%%  An alternative, more efficient way to compute the steady state and the
            %%%%%%%%  matrices GAMMA3, GAMMA4 would use the numerical
            %%%%%%%%  approach programmed in simul_diffus.m, that does not
            %%%%%%%%  need analytical derivatives
              param=[A,B,beta,sigma,n,deltak,deltah,alpha,phi1,phi2,psi];
              luss0=log(u);lxss0=log(x);lzss0=log(z);lthetss0=0;letass0=0;
              X0=[lzss0 lzss0 lxss0 lxss0 luss0 luss0 lthetss0 lthetss0 letass0 letass0]';
              X1=fsolve('ssendog',X0,optimset,param);
              lzss=X1(1);zss=exp(lzss);
              lxss=X1(3);xss=exp(lxss);
              luss=X1(5);uss=exp(luss);
    
              J=jacobian('ssendog',X1,param);
                Mgamma0=-[J(1,1) J(1,3) J(1,5);
                          J(2,1) J(2,3) J(2,5);
                          J(3,1) J(3,3) J(3,5)];
                Mgamma1=[J(1,2) J(1,4) J(1,6);
                         J(2,2) J(2,4) J(2,6);
                         J(3,2) J(3,4) J(3,6)];
                Mgamma2=[J(1,8)+phi1*J(1,7) J(1,10)+phi2*J(1,9);
                         J(2,8)+phi1*J(2,7) J(2,10)+phi2*J(2,9);
                         J(3,8)+phi1*J(3,7) J(3,10)+phi2*J(3,9)];
                     
                Mgamma3=inv(Mgamma0)*Mgamma1;
                Mgamma4=inv(Mgamma0)*Mgamma2;
   %The reader can verify that Mgamma3=GAMMA3 and Mgamma4=GAMMA
Mgamma3
GAMMA3
Mgamma4
GAMMA4

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
  [evec,eval]=eig(GAMMA3);
  
  eval2=diag(eval);

  [eval3,ord]=sort(abs(eval2));

  if eval3(1)>1
     'error - no solution'
  end

  if eval3(2)<1
     ' multiple solutions'
  end

  eval4=eval2(ord);

  Lambda=diag(eval4);

  MM=evec(:,ord);

  Mm=inv(MM);
  
  Q=Mm*GAMMA4;
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if eval3(2)>1
  
  S0=[Mm(2,1);
      Mm(3,1)];
  S1=[Mm(2,2) Mm(2,3);
      Mm(3,2) Mm(3,3)];
  S2=[Q(2,1)/(phi1-Lambda(2,2))   Q(2,2)/(phi2-Lambda(2,2));
      Q(3,1)/(phi1-Lambda(3,3))   Q(3,2)/(phi2-Lambda(3,3))];
  S3=-inv(S1)*S0;
  S4=inv(S1)*S2;
  
  %Simulations
   %initial conditions
   htilde0=1;
   ktilde0=z*htilde0;
   z0=ktilde0/htilde0;
   theta0=1;
   eta0=1;
  
   %shocks
   
   ltheta=[];leta=[];ltheta0=log(theta0);leta0=log(eta0);
   ltheta=[ltheta;ltheta0];leta=[leta;leta0];
   
   for t=1:nobs
       ltheta1=phi1*ltheta0+eps1(t);
       leta1=phi2*leta0+eps2(t);
       ltheta0=ltheta1;
       leta0=leta1;
       ltheta=[ltheta;ltheta0];leta=[leta;leta0];
   end
   
   % z(t), x(t), u(t)
   lz=[]; lz0=log(z0); lz=[lz;lz0];
   for t=1:nobs
       lz1=log(z)+Lambda(1,1)*(lz0-log(z))+...
           ((Q(1,1)+(Lambda(1,1)-phi1)*(S4(1,1)*Mm(1,2)+S4(2,1)*Mm(1,3)))/(Mm(1,1)+Mm(1,2)*S3(1)+Mm(1,3)*S3(2)))*ltheta(t)+...
           ((Q(1,2)+(Lambda(1,1)-phi2)*(S4(1,2)*Mm(1,2)+S4(2,2)*Mm(1,3)))/(Mm(1,1)+Mm(1,2)*S3(1)+Mm(1,3)*S3(2)))*leta(t);
       lz0=lz1;
       lz=[lz;lz0];
   end
   shocks=[ltheta leta]';
   cont=S3*(lz'-log(z))+S4*shocks;
   lx=log(x)+cont(1,:); lx=lx';
   lu=log(u)+cont(2,:); lu=lu';
   
   zt=exp(lz);
   xt=exp(lx);
   ut=exp(lu);
   thetat=exp(ltheta);
   etat=exp(leta);
   
   gammaht=B*(1-ut).*etat-deltah;
   ht=[]; 
   h0=htilde0;
   ht=[ht;h0];  % Detrended stock of human capital 
   for t=1:nobs
       h1=((1+gammaht(t))/(1+gammah))*h0;
       h0=h1;
       ht=[ht;h0];
   end
   kt=ht.*zt;  % Detrended stock of physical capital 
        k0=ktilde0;
   ct=kt.*xt;  % Detrended consumption 
        c0=k0*x;
   gammakt=(1+gamma)*(kt(2:nobs)./kt(1:nobs-1))-1;
   gammact=(1+gamma)*(ct(2:nobs)./ct(1:nobs-1))-1;
   yt=A*(kt.^alpha).*((ut.*ht).^(1-alpha));  % Detrended output 
        y0=A*k0^alpha*(u*h0)^(1-alpha);
   gammayt=(1+gamma)*(yt(2:nobs)./yt(1:nobs-1))-1;
   qt=yt+(1-alpha)*A*thetat.*(1-ut).*(ut.^(-alpha)).*(zt.^alpha).*ht;  % Detrended gross output 
        q0=y0+(1-alpha)*A*(1-u)*(u^(-alpha))*(z^alpha)*h0;
   gammaqt=(1+gamma)*(qt(2:nobs)./qt(1:nobs-1))-1;
   prodt=(1-alpha)*yt./(ut.*ht);
        prod=(1-alpha)*A*(z^alpha)*u^(-alpha);
end
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   
if eval3(2)<1
  
  S1=[Mm(1,1)-(Mm(1,2)*Mm(3,1))/Mm(3,2)   Mm(1,3)-(Mm(1,2)*Mm(3,3)/Mm(3,2));
      Mm(2,1)-(Mm(2,2)*Mm(3,1))/Mm(3,2)   Mm(2,3)-(Mm(2,2)*Mm(3,3)/Mm(3,2))];
  Lambda1=[Lambda(1,1)  0;
           0            Lambda(2,2)];
  S2=[Q(1,1)+(Q(3,1)/(phi1-Lambda(3,3)))*(Mm(1,2)/Mm(3,2))*(Lambda(1,1)-phi1)   Q(1,2)+(Q(3,2)/(phi2-Lambda(3,3)))*(Mm(1,2)/Mm(3,2))*(Lambda(1,1)-phi2)  Mm(1,3)-(Mm(1,2)*Mm(3,3)/Mm(3,2));
      Q(2,1)+(Q(3,1)/(phi1-Lambda(3,3)))*(Mm(2,2)/Mm(3,2))*(Lambda(2,2)-phi1)   Q(2,2)+(Q(3,2)/(phi2-Lambda(3,3)))*(Mm(2,2)/Mm(3,2))*(Lambda(2,2)-phi2)  Mm(2,3)-(Mm(2,2)*Mm(3,3)/Mm(3,2))];
  
  S3=inv(S1)*Lambda1*S1;
  S4=inv(S1)*S2;
  
%  Simulations
%  Initial conditions
   htilde0=1;
   ktilde0=z*htilde0;
   z0=ktilde0/htilde0;
   theta0=1;
   eta0=1;
  
%  Shocks
   
   ltheta=[];leta=[];ltheta0=log(theta0);leta0=log(eta0);
   ltheta=[ltheta;ltheta0];leta=[leta;leta0];
   
   for t=1:nobs
       ltheta1=phi1*ltheta0+eps1(t);
       leta1=phi2*leta0+eps2(t);
       ltheta0=ltheta1;
       leta0=leta1;
       ltheta=[ltheta;ltheta0];leta=[leta;leta0];
   end
   
% z(t), x(t), u(t)
   estad=[];
   shocks=[ltheta(1:nobs) leta(1:nobs) error]';
   lz0=log(z); lu0=log(u0);
   estad0=[lz0;lu0]; estadss=[log(z); log(u)];
   estad=[estad estad0];
   for t=1:nobs
       estad1=estadss+S3*(estad0-estadss)+S4*shocks(:,t);
       estad0=estad1;
       estad=[estad estad0];
   end
   lz=estad(1,:)';
   lu=estad(2,:)';
   lx=log(x)+(1/Mm(3,2))*((Q(3,1)/(phi1-Lambda(3,3)))*ltheta+...
       (Q(3,2)/(phi2-Lambda(3,3)))*leta-Mm(3,1)*(lz-log(z))-...
       Mm(3,3)*(lu-log(u)));
   
   zt=exp(lz);
   xt=exp(lx);
   ut=exp(lu);
   thetat=exp(ltheta);
   etat=exp(leta);
   
   gammaht=B*(1-ut).*etat-deltah;
   ht=[]; 
   h0=htilde0;
   ht=[ht;h0]; % Detrended stock of human capital 
   for t=1:nobs
       h1=((1+gammaht(t))/(1+gammah))*h0;
       h0=h1;
       ht=[ht;h0];
   end
   kt=ht.*zt;  % Detrended stock of physical capital 
        k0=ktilde0;
   ct=kt.*xt;  % Detrended consmption 
        c0=k0*x;
   gammakt=(1+gamma)*(kt(2:nobs)./kt(1:nobs-1))-1;
   gammact=(1+gamma)*(ct(2:nobs)./ct(1:nobs-1))-1;
   yt=A*(kt.^alpha).*((ut.*ht).^(1-alpha)).*(ht.^psi).*thetat;  % output detrended
        y0=A*k0^alpha*(u*h0)^(1-alpha)*h0^psi;
   gammayt=(1+gamma)*(yt(2:nobs)./yt(1:nobs-1))-1;
   qt=yt+(1-alpha)*A*thetat.*(1-ut).*(ut.^(-alpha)).*(zt.^alpha).*(ht.^(1+psi));  % gross output detrended
        q0=y0+(1-alpha)*A*(1-u)*(u^(-alpha))*(z^alpha)*(h0^(1+psi));
   gammaqt=(1+gamma)*(qt(2:nobs)./qt(1:nobs-1))-1;
   prodt=(1-alpha)*yt./(ut.*ht);
        prod=(1-alpha)*A*(z^alpha)*u^(-alpha);
end
       
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

figure;
hold on

subplot(4,3,1)
plot([ht ones(length(ht),1)*htilde0])
title('Detrended human capital ');
xlabel('time');

subplot(4,3,2)
plot([kt ones(length(kt),1)*ktilde0])
title('Detrended physical capital ');
xlabel('time');

subplot(4,3,3)
plot([ut ones(length(ut),1)*u])
title('time used in good final sector');
xlabel('time');

subplot(4,3,4)
plot([yt ones(length(yt),1)*y0])
title('Detrended output ');
xlabel('time');

subplot(4,3,5)
plot([qt ones(length(qt),1)*q0])
title('Detrended gross output ');
xlabel('time');

subplot(4,3,6)
plot([ct ones(length(ct),1)*c0])
title('Detrended consumption ');
xlabel('time');

subplot(4,3,7)
plot([prodt ones(length(prodt),1)*prod])
title('labor productivity');
xlabel('time');

subplot(4,3,8)
plot([gammaht ones(length(gammaht),1)*gammah])
title('growth rate of human capital');
xlabel('time');

subplot(4,3,9)
plot([gammakt ones(length(gammakt),1)*gamma])
title('growth rate of physical capital');
xlabel('time');

subplot(4,3,10)
plot([gammact ones(length(gammact),1)*gamma])
title('growth rate of consumption');
xlabel('time');

subplot(4,3,11)
plot([gammayt ones(length(gammayt),1)*gamma])
title('growth rate of output');
xlabel('time');

subplot(4,3,12)
plot([gammaqt ones(length(gammaqt),1)*gamma])
title('growth rate of gross output');
xlabel('time');

hold off
