% This program computes a single realization of the endogenous growth model
% with human capital accumulation, without externalities in the 
% production of the final good.

% We use the Blanchard-Kahn approach to solve for control variables, but we
% use the original nonlinear model for the variables in levels.

%  @Copyright J. Ruiz 2007 

clear

%  Structural parameters

A=1.0;              %    technology level, final good sector
B=0.0237;           %    technology level, educational sector
beta=0.99;          %    discount factor
sigma=1.5;      	%    inverse of the intertemporal elasticity of substitution (relative risk aversion)
n=0.0035;          	%    population growth rate
deltak=0.025;      	%    depreciation rate of physical capital
deltah=0.008;      	%    depreciation rate of human capital
alpha=0.36;         %    physical capital share (final good sector)
phi1=0.9;           %    persistence parameter of stochastic process for shock in sector of final good
phi2=0.9;           %    persistence parameter of stochastic process for shock in educational sector


nobs=150;           %    number of observations

sigmae1=0.001;      %    standard deviation innovation 1
sigmae2=0.001;      %    standard deviation innovation 2

%  Activate the next two lines when computing impulse responses to a shock in either sector 
%  sigmae1=0.00;      %    standard deviation innovation 1
%  sigmae2=0.00;      %    standard deviation innovation 2

% Innovations

   aleatn=normrnd(0,1,nobs+200,3);
   eps1=sigmae1*aleatn(201:length(aleatn),1); 
   eps2=sigmae2*aleatn(201:length(aleatn),2);

%   eps1(10) = .01;  %  Activate when computing responses to an impulse in the productivity of the final good sector
%   eps2(10) = .01;  %  Activate when computing responses to an impulse in the productivity of the educational sector

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%  Steady state

    gamma=(beta*(B+1-deltah))^(1/sigma)-1;
    u=1-((gamma+deltah)/B);
    z=(A*alpha*(u^(1-alpha))/((((1+gamma)^sigma)*(1+n)/beta)-(1-deltak)))^(1/(1-alpha));
    x=A*z^(alpha-1)*u^(1-alpha)+(1-deltak)-(1+n)*(1+gamma);

         
    GAMMA0=[(1-alpha+sigma)*((1+n)/beta)*(1+gamma)^sigma-(1-alpha)*(1-deltak)     sigma*((1+n)/beta)*(1+gamma)^sigma    -(1-alpha)*((((1+n)/beta)*(1+gamma)^sigma)-(1-deltak));
            -(alpha-sigma)*((1+gamma)^sigma)/beta                                 sigma*((1+gamma)^sigma)/beta          alpha*((1+gamma)^sigma)/beta;
            (1+n)*(1+gamma)*z                                                     0                                     0];
        
    GAMMA1=[sigma*((1+n)/beta)*(1+gamma)^sigma      sigma*((1+n)/beta)*(1+gamma)^sigma   sigma*(1+n)*B*u*((1+gamma)^(sigma-1))/beta;
            -(alpha-sigma)*((1+gamma)^sigma)/beta   sigma*((1+gamma)^sigma)/beta         alpha*((1+gamma)^sigma)/beta+sigma*((1+gamma)^(sigma-1))*B*u/beta;
            ((((1+n)/beta)*(1+gamma)^sigma)-x)*z    -z*x                                 (1+n)*z*B*u+(1-alpha)*A*(z^alpha)*(u^(1-alpha))];
        
    GAMMA2=[((((1+n)/beta)*(1+gamma)^sigma)-(1-deltak))*phi1   -sigma*(1+n)*B*(1-u)*((1+gamma)^(sigma-1))/beta;
            -(((1+gamma)^sigma)/beta)*(1-phi1)                 ((1+gamma)^sigma)/beta-sigma*((1+gamma)^(sigma-1))*B*(1-u)/beta-(1-deltah)*phi2;
            A*(z^alpha)*(u^(1-alpha))                          -(1+n)*z*B*(1-u)];

    GAMMA3=inv(GAMMA0)*GAMMA1;
    
    GAMMA4=inv(GAMMA0)*GAMMA2;    
    
  [evec,eval]=eig(GAMMA3);
  
  eval2=diag(eval);

  [eval3,ord]=sort(abs(eval2));

  if eval3(1)>1
     'error - no solution'
  end


  eval4=eval2(ord);

  Lambda=diag(eval4);

  MM=evec(:,ord);

  Mm=inv(MM);
  
  Q=Mm*GAMMA4;
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  
  S0=[Mm(2,1);
      Mm(3,1)];
  S1=[Mm(2,2) Mm(2,3);
      Mm(3,2) Mm(3,3)];
  S2=[Q(2,1)/(phi1-Lambda(2,2))   Q(2,2)/(phi2-Lambda(2,2));
      Q(3,1)/(phi1-Lambda(3,3))   Q(3,2)/(phi2-Lambda(3,3))];
  S3=-inv(S1)*S0;
  S4=inv(S1)*S2;
  
%  Simulations
%  Initial conditions

   htilde0=1;
   ktilde0=z*htilde0;
   theta0=1;
   eta0=1;
   
%  shocks

ltheta=[];leta=[];ltheta0=log(theta0);leta0=log(eta0);
   ltheta=[ltheta;ltheta0];leta=[leta;leta0];
   
   for t=1:nobs
       ltheta1=phi1*ltheta0+eps1(t);
       leta1=phi2*leta0+eps2(t);
       ltheta0=ltheta1;
       leta0=leta1;
       ltheta=[ltheta;ltheta0];leta=[leta;leta0];
   end
   
   k0=ktilde0;
   h0=htilde0;
   ct=[];
   ht=[];
   kt=[];
   ut=[];
   kt=[kt;k0];
   ht=[ht;h0];
   for t=1:nobs
       z0=k0/h0;
       c0=exp(log(k0)+log(x)+S3(1)*log(z0/z)+S4(1,1)*ltheta(t)+S4(1,2)*leta(t));
       ct=[ct;c0];
       u0=exp(log(u)+S3(2)*log(z0/z)+S4(2,1)*ltheta(t)+S4(2,2)*leta(t));
       ut=[ut;u0];
       k1=(1/((1+n)*(1+gamma)))*(A*(k0^alpha)*((u0*h0)^(1-alpha))*...
           exp(ltheta(t))+(1-deltak)*k0-c0);
       h1=(1/(1+gamma))*(B*(1-u0)*h0*exp(leta(t))+(1-deltah)*h0);
       k0=k1;
       h0=h1;
       kt=[kt;k0];
       ht=[ht;h0];
   end
   
   
   thetat=exp(ltheta);
   etat=exp(leta);
   gamht=(1+gamma)*(ht(2:nobs)./ht(1:nobs-1))-1;
   gammaht=B*(1-ut).*etat(1:nobs)-deltah;
   gammakt=(1+gamma)*(kt(2:nobs)./kt(1:nobs-1))-1;
   gammact=(1+gamma)*(ct(2:nobs)./ct(1:nobs-1))-1;
   yt=A*(kt(1:nobs).^alpha).*((ut.*ht(1:nobs)).^(1-alpha)).*thetat(1:nobs);  % output detrended
        y0=A*k0^alpha*(u*h0)^(1-alpha);
   gammayt=(1+gamma)*(yt(2:nobs)./yt(1:nobs-1))-1;
   qt=yt+(1-alpha)*A*thetat(1:nobs).*(1-ut).*(ut.^(-alpha)).*((kt(1:nobs)./ht(1:nobs)).^alpha).*ht(1:nobs);  % gross output detrended
        q0=y0+(1-alpha)*A*(1-u)*(u^(-alpha))*(z^alpha)*h0;
   gammaqt=(1+gamma)*(qt(2:nobs)./qt(1:nobs-1))-1;
   prodt=(1-alpha)*A*((kt(1:nobs)./ht(1:nobs)).^alpha).*ut.^(-alpha).*thetat(1:nobs);
        prod=(1-alpha)*A*(z^alpha)*u^(-alpha);

       
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

figure;
hold on

subplot(4,3,1)
plot([ht ones(length(ht),1)*htilde0])
title('Detrended human capital ');
xlabel('time');

subplot(4,3,2)
plot([kt ones(length(kt),1)*ktilde0])
title('Detrended physical capital');
xlabel('time');

subplot(4,3,3)
plot([ut ones(length(ut),1)*u])
title('time used in final good sector');
xlabel('time');

subplot(4,3,4)
plot([yt ones(length(yt),1)*y0])
title('Detrended output ');
xlabel('time');

subplot(4,3,5)
plot([qt ones(length(qt),1)*q0])
title('Detrended gross output ');
xlabel('time');

subplot(4,3,6)
plot([ct ones(length(ct),1)*c0])
title('Detrended consumption ');
xlabel('time');

subplot(4,3,7)
plot([prodt ones(length(prodt),1)*prod])
title('labor productivity');
xlabel('time');

subplot(4,3,8)
plot([gammaht ones(length(gammaht),1)*gamma])
title('growth rate of human capital');
xlabel('time');

subplot(4,3,9)
plot([gammakt ones(length(gammakt),1)*gamma])
title('growth rate of physical capital');
xlabel('time');

subplot(4,3,10)
plot([gammact ones(length(gammact),1)*gamma])
title('growth rate of consumption');
xlabel('time');

subplot(4,3,11)
plot([gammayt ones(length(gammayt),1)*gamma])
title('growth rate of output');
xlabel('time');

subplot(4,3,12)
plot([gammaqt ones(length(gammaqt),1)*gamma])
title('growth rate of gross output');
xlabel('time');

hold off
