% This program computes the steady-state of an economy with human capital 
% accumulation, with no externalities, leisure as an argument in the utility
% function and physical capital entering as an input in the educational
% sector.

% The program incorporates consumption taxes as well as taxes on labor and 
% capital income obtained from working in the sector producing the final good.
% Steady-state effects of changes in the tax rate on labor income are displayed.

% @Copyright J. Ruiz 2007

clear

%  structural parameters

A=1.0;              %    technology level, final good sector
B=0.03942;          %    technology level, educational sector
rho= 0.99;      	%    discount factor
sigma=1.5;      	%    inverse of the intertemporal elasticity of substitution (relative risk aversion)
n=0.0035;          	%    population growth rate
deltak=0.025;      	%    depreciation rate of physical capital
deltah=0.008;      	%    depreciation rate of human capital
alpha=0.36;         %    physical capital elasticity (final good sector)
beta=0.15;          %    physical capital elasticity (educational sector)
p=1/3;              %    weight of leisure in utility function
tauc=0;         	%    consumption tax rate
taur=0;         	%    capital income tax rate
tauw=0;             %    labor income tax rate


%  steady state
    
    z0=10.46; x0=0.056; u0=0.26; v0=0.8; l0=0.52; gamma0=0.0037;
    
    x0=[z0 x0 u0 v0 l0 gamma0]';
    
    vtauw=[];
    vz=[];
    vx=[];
    vu=[];
    vv=[];
    vl=[];
    vgamma=[];
    
    for i=1:100
        tauw=(i-1)/100;
        param=[A B rho sigma n deltak deltah alpha beta p tauc taur tauw];
        x1=fsolve('sslucas',x0,optimset,param);
        x0=x1;
        vtauw=[vtauw;tauw];
        vz=[vz;x1(1)];
        vx=[vx;x1(2)];
        vu=[vu;x1(3)];
        vv=[vv;x1(4)];
        vl=[vl;x1(5)];
        vgamma=[vgamma;x1(6)];
        if x1(6)<0 
            break
        end
      
    end
    
    vyk=A*(vv.^alpha).*(vz.^(alpha-1)).*(vu.^(1-alpha)); % output to physical capital ratio
    vqk=vyk+((1-taur)*alpha*A.*(vyk./vv)+taur*deltak).*((1-vv)/beta); % gross output to physical capital raio
    vgy=(taur*alpha+vtauw*(1-alpha))-taur*deltak*(vv./vyk)+tauc.*(vx./vyk); % public revenues to output ratio
    vgq=(taur*alpha+vtauw*(1-alpha)).*(vyk./vqk)-taur*deltak*(vv./vqk)+tauc.*(vx./vqk); % public revenues to gross output ratio 
    vstudy=1-vu-vl; % fraction of time used in education
    
    figure;
    hold on
    
    subplot(4,3,1);
    plot(vtauw,vx);
    title('effects on x');
    xlabel('labor income tax rate');
    ylabel('x');
    
    subplot(4,3,2);
    plot(vtauw,vz);
    title('effects on z');
    xlabel('labor income tax rate');
    ylabel('z');
    
    subplot(4,3,3);
    plot(vtauw,vu);
    title('effects on u');
    xlabel('labor income tax rate');
    ylabel('u');
    
    subplot(4,3,4);
    plot(vtauw,vv);
    title('effects on v');
    xlabel('labor income tax rate');
    ylabel('v');
    
    subplot(4,3,5);
    plot(vtauw,vl);
    title('effects on l');
    xlabel('labor income tax rate');
    ylabel('l');
    
    subplot(4,3,6);
    plot(vtauw,vgamma);
    title('effects on growth rate');
    xlabel('labor income tax rate');
    ylabel('growth rate');
    
    subplot(4,3,7);
    plot(vtauw,vyk);
    title('effects on output to capital ratio');
    xlabel('labor income tax rate');
    ylabel('y/k');
    
    subplot(4,3,8);
    plot(vtauw,vqk);
    title('effects on gross output to capital ratio');
    xlabel('labor income tax rate');
    ylabel('q/k');
    
    subplot(4,3,9);
    plot(vtauw,vstudy);
    title('effects on time used in education');
    xlabel('labor income tax rate');
    ylabel('1-u-l');
    
    subplot(4,3,10);
    plot(vtauw,vgy);
    title('effects on public revenues to output ratio');
    xlabel('labor income tax rate');
    ylabel('g/y');
    
    subplot(4,3,11);
    plot(vtauw,vgq);
    title('effects on public revenues to gross output ratio');
    xlabel('labor income tax rate');
    ylabel('g/q');
    
    hold off
    
    