% This program computes the transmission of a shock in the leader country to 
% the follower country. We do that by computing impulse responses in both
% countries to a productivity shock in the Leader country.
%
% The program can also be used to generate a realization of the numerical 
% solution to the problems of the leader and the follower countries, given
% a realization of the stochastic shocks for both countries.

% @Copyright J. Ruiz 2007

clear;

% Structural parameters

% The standard deviation of innovations productivity in both countries is
% set to zero when the program is used to compute impulse responses
% Those lines should be commented, and nonzero values given to the
% standard deviations of innovations, when computing a solution realization. 

    % Common Parameters 
    sigma = 1.5;        %   inverse of the intertemporal elasticity of substitution (relative risk aversion)
    alpha = 0.36;       %   physical capital share (final good sector)
    beta = 0.99;        %   discount factor
    nobs = 150;         %   number of observations
    % Leader country parameters
    A1 = 1;             %   technology level, final good sector 
    L1 = 1;             %   population size
    eta1 = 1;           %   innovation cost
    phi1 = 0.90;        %   persistence parameter of stochastic process for shock in sector of final good
    sigmae1 = 0.0;      %   standard deviation of the innovation in stochastic process for theta
%    sigmae1 = .01;      %   Activate when computing a sample realization
    % Follower country parameters
    A2 = 0.9;           %   technology level, final good sector 
    L2 = 1;             %   population size
    eta2 = 1;           %   innovation cost
    phi2 = 0.90;        %   persistence parameter of stochastic process for shock in sector of final good
    sigmae2 = 0.0;      %   standard deviation of the innovation in stochastic process for theta
%    sigmae2 = .01;      %   Activate when computing a sample realization
    b = 0.5;            %   elasticity of the imitation-cost function

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Realization for innovations
   aleatn=normrnd(0,1,nobs+200,3);

   eps1 = sigmae1*aleatn(201:length(aleatn),1); %   leader country
   eps2 = sigmae2*aleatn(201:length(aleatn),2); %   follower country

% The following line introduces an impulse shock of size .10 in country 1 at time 10
% The line must be commented if the program is used to compute solution
% realizations
   eps1(10)=0.1;   % shock in period 10

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Shocks in Leader country 
    thetass1= 1;
    theta10 = thetass1;
    theta1=[]; theta1=[theta1; theta10];
    for t=1:nobs
        theta11=exp(phi1*log(theta10)+eps1(t));
        theta10=theta11;
        theta1=[theta1; theta10];
    end
    mu1=theta1.^(1/(1-alpha));
    
%   Computing paths for variables in leader country
%   We only compute detrended variables and variables without
%   growth
   
    Ap1=(A1^(1/(1-alpha)))*(alpha^(alpha/(1-alpha)))*(1-alpha)/(eta1/L1);
    gammass1=(beta*(1+Ap1*(alpha^(1/(1-alpha)))))^(1/sigma)-1;
    z1ss=Ap1*(alpha^(alpha/(1-alpha)))*(1+alpha)-gammass1;
    varphi1=(1+gammass1)/(1+Ap1*(alpha^(alpha/(1-alpha)))*(1+alpha));
    varphi2=Ap1*(alpha^(alpha/(1-alpha)))*(sigma*((1+gammass1)^(sigma-1))*(1+alpha)-beta)/...
        (sigma*((1+gammass1)^sigma)*varphi1);
    lz1=log(z1ss)+(varphi2/(1-varphi1))*log(mu1);
    z1=exp(lz1);
    Nt10=10;
    qt10=(eta1/L1)*Nt10; q10=qt10;
    q1=[]; q1=[q1;q10];
    for t=1:nobs
        q11=(1/(1+gammass1))*(mu1(t)*Ap1*(alpha^(alpha/(1-alpha)))*(1+alpha)+1-z1(t))*q10;
        q10=q11;
        q1=[q1;q10];
    end
    c1=z1.*q1;
    N1=(L1/eta1)*q1;
    r1=Ap1*(alpha^(1/(1-alpha)))*mu1;
    y1=(A1^(1/(1-alpha)))*(alpha^(2*alpha/(1-alpha)))*mu1.*N1;
    Y1=L1*y1;
    w1=(1-alpha)*y1;
    X1=L1*(A1^(1/(1-alpha)))*(alpha^(2/(1-alpha)))*mu1;
    pi1=((1-alpha)/alpha)*(alpha^(2/(1-alpha)))*(A1^(1/(1-alpha)))*L1*mu1;
    
% Shocks in Follower country 
    thetass2 = 1;
    theta20 = thetass2;
    theta2=[]; theta2=[theta2; theta20];
    for t=1:nobs
        theta21=exp(phi2*log(theta20)+eps2(t));
        theta20=theta21;
        theta2=[theta2; theta20];
    end
    mu2=theta2.^(1/(1-alpha));
    
%   Computing paths for variables in Follower country 
%   We only compute detrended variables and variables without
%   growth

    Ap2=(1-alpha^2)*(A2^(1/(1-alpha)))*(alpha^(2*alpha/(1-alpha)))*L2;
    v2ss=(A2/A1)^(1/(1-alpha))*(L2/L1)*eta1;
    rc2N2ss=((1-alpha^2)*(A2^(1/(1-alpha)))*(alpha^(2*alpha/(1-alpha)))+...
        (v2ss/L2))-(v2ss/L2)*(1+gammass1);
    rN2N1ss=(v2ss/eta2)^(1/b);
    ry2y1ss=rN2N1ss*(A2/A1)^(1/(1-alpha));
    z2ss=rc2N2ss*(L2/v2ss);
    
    param=[Ap1,Ap2,sigma,beta,alpha,b,log(z1ss),...
        (1/(1-alpha))*log(thetass1),(1/(1-alpha))*log(thetass2)];
    X0=[log(z2ss),log(z2ss),log(v2ss),log(v2ss),...
        (1/(1-alpha))*log(thetass2),(1/(1-alpha))*log(thetass2),...
        log(z1ss),(1/(1-alpha))*log(thetass1)]';
    %X=fsolve('ssdiff',X0,optimset,param);
    
    %  The next block of instructions calls the Jacobian.m function to
    %  compute numerical partial derivatives in the log-linear
    %  approximation
    
    J=jacobian('ssdiff',X0,param);
    GAMMA0=[J(1,1) J(1,3);
            J(2,1) J(2,3)];
    GAMMA1=[J(1,2) J(1,4);
            J(2,2) J(2,4)];
    GAMMA2=[J(1,5)*phi2+J(1,6) J(1,7) J(1,8);
            J(2,6)             J(2,7) J(2,8)];
    GAMMA3=-inv(GAMMA0)*GAMMA1;
    GAMMA4=-inv(GAMMA0)*GAMMA2;
    [evec,eval]=eig(GAMMA3);
  
  eval2=diag(eval);

  [eval3,ord]=sort(abs(eval2));

  eval4=eval2(ord);

  Lambda=diag(eval4);
  
  MM=evec(:,ord);

  Mm=inv(MM);
  
  Q=Mm*GAMMA4;
  
  S1=(Q(1,3)+Q(1,2)*varphi2/(1-varphi1))+(Lambda(1,1)-phi1)*...
      (Mm(1,1)/Mm(2,1))*(Q(2,3)+Q(2,2)*(varphi2/(1-varphi1)))/...
      (phi1-Lambda(2,2));
  S2=Q(1,1)+(Lambda(1,1)-phi2)*(Mm(1,1)/Mm(2,1))*Q(2,1)/...
      (phi2-Lambda(2,2));
  
  Nt20=((v2ss/eta2)^(1/b))*Nt10;
  v20=eta2*(Nt20/Nt10)^b;
  lv20=log(v20);
  lv2=[];lv2=[lv2;lv20];
  for t=1:nobs
      lv21=log(v2ss)+Lambda(1,1)*(lv20-log(v2ss))+...
          (Mm(2,1)/(Mm(1,2)*Mm(2,1)-Mm(1,1)*Mm(2,2)))*...
          (S1*log(mu1(t))+S2*log(mu2(t)));
      lv20=lv21;
      lv2=[lv2;lv20];
  end
  v2=exp(lv2);
  lz2=log(z2ss)+(1/Mm(2,1))*...
      (((Q(2,3)+Q(2,2)*varphi2/(1-varphi1))/(phi1-Lambda(2,2)))*log(mu1)+...
      (Q(2,1)/(phi2-Lambda(2,2)))*log(mu2)-Mm(2,2)*(lv2-log(v2ss)));
  z2=exp(lz2);
  q2=(L1/(L2*eta1*eta2^(1/b)))*q1.*(v2.^((1+b)/b));
  c2=q2.*z2;
  N2=L2*(q2./v2);
  dv2=(v2(2:length(v2))-v2(1:length(v2)-1))./v2(1:length(v2)-1);
  r2=(alpha/(1+alpha))*Ap2*(mu2(1:length(mu2)-1)./v2(1:length(v2)-1))+dv2;
  X2=L2*(Ap2^(1/(1-alpha)))*(alpha^(2/(1-alpha)))*mu2;
  Y2=(1/alpha^2)*N2.*X2;
  y2=Y2/L2;
  pi2=((1-alpha)/alpha)*(alpha^(2/(1-alpha)))*(A2^(1/(1-alpha)))*L2*mu2;
  w2=(1-alpha)*y2;
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%%%                 Graphs                   %%%%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  time=1:1:nobs+1;
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  figure;
  hold on
  axis('tight');
  
  subplot(5,2,1)
  plot(c1);
  title('detrended leader country consumption');
  xlabel('time');
  
  
  subplot(5,2,3)
  plot(y1);
  title('detrended leader country output');
  xlabel('time');
  
  
  subplot(5,2,5)
  plot(N1);
  title('detrended leader country number of varieties');
  xlabel('time');
  
  
  subplot(5,2,7)
  plot(w1);
  title('detrended leader country wage');
  xlabel('time');
  
    
  subplot(5,2,9)
  plot(r1);
  title('leader country real interest rate');
  xlabel('time');
  
  
  subplot(5,2,2)
  plot(c2);
  title('detrended follower country consumption');
  xlabel('time');
 
  subplot(5,2,4)
  plot(y2);
  title('detrended follower country output');
  xlabel('time');
  
  subplot(5,2,6)
  plot(N2);
  title('detrended follower country number of varieties');
  xlabel('time');

  subplot(5,2,8)
  plot(w2);
  title('detrended follower country wage');
  xlabel('time');

  subplot(5,2,10)
  plot(r2);
  title('follower country real interest rate');
  xlabel('time');
 
  
  hold off
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  figure;
  hold on
  
  subplot(4,2,1)
  plot((1+gammass1)*c1(2:length(c1))./c1(1:length(c1)-1)-1);
  title('leader country consumption growth rates');
  
  subplot(4,2,2)
  plot((1+gammass1)*c2(2:length(c2))./c2(1:length(c2)-1)-1);
  title('follower country consumption growth rates');
  
  subplot(4,2,3)
  plot((1+gammass1)*y1(2:length(y1))./y1(1:length(y1)-1)-1);
  title('leader country output growth rates');
  
  subplot(4,2,4)
  plot((1+gammass1)*y2(2:length(y2))./y2(1:length(y2)-1)-1);
  title('follower country output growth rates');
  
  subplot(4,2,5)
  plot((1+gammass1)*N1(2:length(N1))./N1(1:length(N1)-1)-1);
  title('leader country growth rate in number of varieties ');
  
  subplot(4,2,6)
  plot((1+gammass1)*N2(2:length(N2))./N2(1:length(N2)-1)-1);
  title('follower country growth rate in number of varieties ');
  
  subplot(4,2,7)
  plot((1+gammass1)*w1(2:length(w1))./w1(1:length(w1)-1)-1);
  title('leader country wage growth rates');
  
  subplot(4,2,8)
  plot((1+gammass1)*w2(2:length(w2))./w2(1:length(w2)-1)-1);
  title('follower country wage growth rates');
  
  hold off
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
  figure;
  hold on
 
  subplot(5,2,1)
  [AX,H1,H2]=plotyy(time,c1,time,c2,'plot');
  set(get(AX(1),'Ylabel'),'String','leader country');
  set(get(AX(2),'Ylabel'),'String','follower country');
  title('detrended consumption per capita');
  %xlabel('time');
  %legend([H1,H2],'leader country','follower country');
  
  subplot(5,2,3)
  [AX,H1,H2]=plotyy(time,y1,time,y2,'plot');
  set(get(AX(1),'Ylabel'),'String','leader country');
  set(get(AX(2),'Ylabel'),'String','follower country');
  title('detrended output per capita');
  %xlabel('time');
  %legend([H1,H2],'leader country','follower country');
  
  subplot(5,2,5)
  [AX,H1,H2]=plotyy(time,N1,time,N2,'plot');
  set(get(AX(1),'Ylabel'),'String','leader country');
  set(get(AX(2),'Ylabel'),'String','follower country');
  title('detrended number of varieties');
  %xlabel('time');
  %legend([H1,H2],'leader country','follower country');
  
  subplot(5,2,9)
  [AX,H1,H2]=plotyy(time(1:length(r2)),r1(1:length(r2)),time(1:length(r2)),r2,'plot');
  set(get(AX(1),'Ylabel'),'String','leader country');
  set(get(AX(2),'Ylabel'),'String','follower country');
  title('real interest rates');
  %xlabel('time');
  %legend([H1,H2],'leader country','follower country');
  
  subplot(5,2,7)
  [AX,H1,H2]=plotyy(time,w1,time,w2,'plot');
  set(get(AX(1),'Ylabel'),'String','leader country');
  set(get(AX(2),'Ylabel'),'String','follower country');
  title('detrended wage');
  %xlabel('time');
  %legend([H1,H2],'leader country','follower country');
  
  subplot(5,2,2)
  [AX,H1,H2]=plotyy(time(1:length(r2)),(1+gammass1)*c1(2:length(c1))./c1(1:length(c1)-1)-1,...
      time(1:length(r2)),(1+gammass1)*c2(2:length(c2))./c2(1:length(c2)-1)-1,'plot');
  set(get(AX(1),'Ylabel'),'String','leader country');
  set(get(AX(2),'Ylabel'),'String','follower country');
  title('consumption growth rates');
  %xlabel('time');
  %legend([H1,H2],'leader country','follower country');
  
  subplot(5,2,4)
  [AX,H1,H2]=plotyy(time(1:length(r2)),(1+gammass1)*y1(2:length(y1))./y1(1:length(y1)-1)-1,...
      time(1:length(r2)),(1+gammass1)*y2(2:length(y2))./y2(1:length(y2)-1)-1,'plot');
  set(get(AX(1),'Ylabel'),'String','leader country');
  set(get(AX(2),'Ylabel'),'String','follower country');
  title('output growth rates');
  %xlabel('time');
  %legend([H1,H2],'leader country','follower country');
   
  subplot(5,2,6)
  [AX,H1,H2]=plotyy(time(1:length(r2)),(1+gammass1)*N1(2:length(N1))./N1(1:length(N1)-1)-1,...
      time(1:length(r2)),(1+gammass1)*N2(2:length(N2))./N2(1:length(N2)-1)-1,'plot');

  set(get(AX(1),'Ylabel'),'String','leader country');
  set(get(AX(2),'Ylabel'),'String','follower country');
  title('number of varieties growth rates');
  %xlabel('time');
  %legend([H1,H2],'leader country','follower country');
  
  subplot(5,2,8)
  [AX,H1,H2]=plotyy(time(1:length(r2)),(1+gammass1)*w1(2:length(w1))./w1(1:length(w1)-1)-1,...
      time(1:length(r2)),(1+gammass1)*w2(2:length(w2))./w2(1:length(w2)-1)-1,'plot');
  set(get(AX(1),'Ylabel'),'String','leader country');
  set(get(AX(2),'Ylabel'),'String','follower country');
  title('wage growth rates');
  %xlabel('time');
  %legend([H1,H2],'leader country','follower country');
  
  subplot(5,2,10)
  plot(v2)
  title('imitation cost');
  
  hold off
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%